(define /package/prog/prjlibs/load
  (let ((loaded-libs (make-hash-table 127)))
    (define (/package/prog/prjlibs/load path . rest)
      "\
If PATH has not yet been loaded, a new module is created, and the PRELOADS
and the file PATH are loaded into it.  IMPORTS are extracted from PATH's
module into the current module, and a copy of PATH's module is returned."
      (let* ((rest-length (length rest))
             (imports  (if (> rest-length 0) (car  rest) '()))
             (preloads (if (> rest-length 1) (cadr rest) '()))
             (handle (hash-create-handle! loaded-libs path #f))
             (callee (cdr handle))
             (caller (current-module)))
        (define (preload obj)
          (if (symbol? obj)
            (module-define! callee obj       (module-ref caller obj))
            (module-define! callee (car obj) (cadr obj))))
        (define (import obj)
           (if (symbol? obj)
             (module-define! caller obj       (module-ref callee obj))
             (module-define! caller (car obj) (module-ref callee (cadr obj)))))
        (if (not callee)
          (let ((port (open path (logior O_RDONLY O_NDELAY))))
            (if (eq? 'directory (stat:type (stat port)))
              (throw '/package/prog/prjlibs/load 'directory path))
            (set! callee (make-module 31 `(,(resolve-interface '(guile)))))
            (set-module-name! callee `(,(string->symbol path)))
            (preload `(/package/prog/prjlibs/load ,/package/prog/prjlibs/load))
            (for-each preload preloads)
            (let loop ()
              (let ((form (read port)))
                (if (eof-object? form)
                  #f
                  (begin
                    (eval form callee)
                    (loop)))))
            (close-port port)
            (set-cdr! handle callee)))
        (for-each import imports)
        (let ((new-module (make-module 31)))
          (define (copy-var sym var)
            (if (or (eq? sym '/package/prog/prjlibs/load)
                    (not (variable-bound? var)))
              #f
              (module-define! new-module sym (variable-ref var))))
          (module-for-each copy-var callee)
          (set-module-name! new-module (module-name callee))
          new-module)))
    /package/prog/prjlibs/load))
